@@ -60,7 +60,7 @@ ppc64_adjust_section (DSO *dso, int n, GElf_Addr start, GElf_Addr adjust)
 
 static int
 ppc64_adjust_dyn (DSO *dso, int n, GElf_Dyn *dyn, GElf_Addr start,
-		 GElf_Addr adjust)
+		  GElf_Addr adjust)
 {
   if (dyn->d_tag == DT_PPC64_GLINK && dyn->d_un.d_ptr >= start)
     {
@@ -73,7 +73,7 @@ ppc64_adjust_dyn (DSO *dso, int n, GElf_Dyn *dyn, GElf_Addr start,
 
 static int
 ppc64_adjust_rel (DSO *dso, GElf_Rel *rel, GElf_Addr start,
-		 GElf_Addr adjust)
+		  GElf_Addr adjust)
 {
   error (0, 0, "%s: PowerPC64 doesn't support REL relocs", dso->filename);
   return 1;
@@ -81,7 +81,7 @@ ppc64_adjust_rel (DSO *dso, GElf_Rel *rel, GElf_Addr start,
 
 static int
 ppc64_adjust_rela (DSO *dso, GElf_Rela *rela, GElf_Addr start,
-		  GElf_Addr adjust)
+		   GElf_Addr adjust)
 {
   if (GELF_R_TYPE (rela->r_info) == R_PPC64_RELATIVE)
     {
@@ -306,7 +306,7 @@ ppc64_prelink_rela (struct prelink_info *info, GElf_Rela *rela,
 
 static int
 ppc64_apply_conflict_rela (struct prelink_info *info, GElf_Rela *rela,
-			  char *buf)
+			   char *buf, GElf_Addr dest_addr)
 {
   switch (GELF_R_TYPE (rela->r_info))
     {
@@ -439,7 +439,8 @@ ppc64_prelink_conflict_rela (DSO *dso, struct prelink_info *info,
   int r_type;
 
   if (GELF_R_TYPE (rela->r_info) == R_PPC64_RELATIVE
-      || GELF_R_TYPE (rela->r_info) == R_PPC64_NONE)
+      || GELF_R_TYPE (rela->r_info) == R_PPC64_NONE
+      || info->dso == dso)
     /* Fast path: nothing to do.  */
     return 0;
   conflict = prelink_conflict (info, GELF_R_SYM (rela->r_info),
@@ -463,6 +464,12 @@ ppc64_prelink_conflict_rela (DSO *dso, struct prelink_info *info,
 	}
       value = 0;
     }
+  else if (conflict->ifunc)
+    {
+      error (0, 0, "%s: STT_GNU_IFUNC not handled on PowerPC64 yet",
+	     dso->filename);
+      return 1;
+    }
   else
     {
       /* DTPREL wants to see only real conflicts, not lookups
