@@ -67,7 +67,7 @@ s390x_adjust_dyn (DSO *dso, int n, GElf_Dyn *dyn, GElf_Addr start,
 
 static int
 s390x_adjust_rel (DSO *dso, GElf_Rel *rel, GElf_Addr start,
-		   GElf_Addr adjust)
+		  GElf_Addr adjust)
 {
   error (0, 0, "%s: S390 doesn't support REL relocs", dso->filename);
   return 1;
@@ -75,7 +75,7 @@ s390x_adjust_rel (DSO *dso, GElf_Rel *rel, GElf_Addr start,
 
 static int
 s390x_adjust_rela (DSO *dso, GElf_Rela *rela, GElf_Addr start,
-		  GElf_Addr adjust)
+		   GElf_Addr adjust)
 {
   Elf64_Addr addr;
 
@@ -108,7 +108,7 @@ s390x_prelink_rel (struct prelink_info *info, GElf_Rel *rel, GElf_Addr reladdr)
 
 static int
 s390x_prelink_rela (struct prelink_info *info, GElf_Rela *rela,
-		   GElf_Addr relaaddr)
+		    GElf_Addr relaaddr)
 {
   DSO *dso = info->dso;
   GElf_Addr value;
@@ -191,7 +191,7 @@ s390x_prelink_rela (struct prelink_info *info, GElf_Rela *rela,
 
 static int
 s390x_apply_conflict_rela (struct prelink_info *info, GElf_Rela *rela,
-			  char *buf)
+			   char *buf, GElf_Addr dest_addr)
 {
   switch (GELF_R_TYPE (rela->r_info))
     {
@@ -276,7 +276,7 @@ s390x_apply_rela (struct prelink_info *info, GElf_Rela *rela, char *buf)
 
 static int
 s390x_prelink_conflict_rel (DSO *dso, struct prelink_info *info, GElf_Rel *rel,
-			   GElf_Addr reladdr)
+			    GElf_Addr reladdr)
 {
   error (0, 0, "%s: S390 doesn't support REL relocs", dso->filename);
   return 1;
@@ -284,7 +284,7 @@ s390x_prelink_conflict_rel (DSO *dso, struct prelink_info *info, GElf_Rel *rel,
 
 static int
 s390x_prelink_conflict_rela (DSO *dso, struct prelink_info *info,
-			    GElf_Rela *rela, GElf_Addr relaaddr)
+			     GElf_Rela *rela, GElf_Addr relaaddr)
 {
   GElf_Addr value;
   struct prelink_conflict *conflict;
@@ -293,7 +293,8 @@ s390x_prelink_conflict_rela (DSO *dso, struct prelink_info *info,
   int r_type;
 
   if (GELF_R_TYPE (rela->r_info) == R_390_RELATIVE
-      || GELF_R_TYPE (rela->r_info) == R_390_NONE)
+      || GELF_R_TYPE (rela->r_info) == R_390_NONE
+      || info->dso == dso)
     /* Fast path: nothing to do.  */
     return 0;
   conflict = prelink_conflict (info, GELF_R_SYM (rela->r_info),
@@ -313,6 +314,12 @@ s390x_prelink_conflict_rela (DSO *dso, struct prelink_info *info,
 	}
       value = 0;
     }
+  else if (conflict->ifunc)
+    {
+      error (0, 0, "%s: STT_GNU_IFUNC not handled on S390x yet",
+	     dso->filename);
+      return 1;
+    }
   else
     {
       /* DTPOFF wants to see only real conflicts, not lookups
