@@ -30,7 +30,7 @@
 
 static int
 sparc_adjust_dyn (DSO *dso, int n, GElf_Dyn *dyn, GElf_Addr start,
-		 GElf_Addr adjust)
+		  GElf_Addr adjust)
 {
   if (dyn->d_tag == DT_PLTGOT)
     {
@@ -55,7 +55,7 @@ sparc_adjust_dyn (DSO *dso, int n, GElf_Dyn *dyn, GElf_Addr start,
 
 static int
 sparc_adjust_rel (DSO *dso, GElf_Rel *rel, GElf_Addr start,
-		 GElf_Addr adjust)
+		  GElf_Addr adjust)
 {
   error (0, 0, "%s: Sparc doesn't support REL relocs", dso->filename);
   return 1;
@@ -63,7 +63,7 @@ sparc_adjust_rel (DSO *dso, GElf_Rel *rel, GElf_Addr start,
 
 static int
 sparc_adjust_rela (DSO *dso, GElf_Rela *rela, GElf_Addr start,
-		  GElf_Addr adjust)
+		   GElf_Addr adjust)
 {
   if (GELF_R_TYPE (rela->r_info) == R_SPARC_RELATIVE)
     {
@@ -235,7 +235,7 @@ sparc_prelink_rela (struct prelink_info *info, GElf_Rela *rela,
 
 static int
 sparc_apply_conflict_rela (struct prelink_info *info, GElf_Rela *rela,
-			  char *buf)
+			   char *buf, GElf_Addr dest_addr)
 {
   switch (GELF_R_TYPE (rela->r_info))
     {
@@ -333,7 +333,8 @@ sparc_prelink_conflict_rela (DSO *dso, struct prelink_info *info,
   int r_type;
 
   if (GELF_R_TYPE (rela->r_info) == R_SPARC_RELATIVE
-      || GELF_R_TYPE (rela->r_info) == R_SPARC_NONE)
+      || GELF_R_TYPE (rela->r_info) == R_SPARC_NONE
+      || info->dso == dso)
     /* Fast path: nothing to do.  */
     return 0;
   conflict = prelink_conflict (info, GELF_R_SYM (rela->r_info),
@@ -355,6 +356,12 @@ sparc_prelink_conflict_rela (DSO *dso, struct prelink_info *info,
 	}
       value = 0;
     }
+  else if (conflict->ifunc)
+    {
+      error (0, 0, "%s: STT_GNU_IFUNC not handled on SPARC yet",
+	     dso->filename);
+      return 1;
+    }
   else
     {
       /* DTPOFF32 wants to see only real conflicts, not lookups
