@@ -18,4 +18,5 @@ int main()
 
 asm (".section nonalloced,\"aw\",@nobits\n\t"
      ".globl testzero\n\t"
-     "testzero: .skip 16384");
+     "testzero: .skip 16384\n\t"
+     ".previous");
